<?php

/**
 * @file
 * Functions used to run a function in the background.
 */

/**
 * Menu Callback; run given function.
 */
function httprl_async_page() {
  // Exit if
  //  The master_key is not set.
  //  The temp_key is not set.
  //  The temp_key does not start with httprl_.
  //  The master_key does not match the md5 of drupal_get_private_key().
  if (   empty($_POST['master_key'])
      || empty($_POST['temp_key'])
      || strpos($_POST['temp_key'], 'httprl_') !== 0
      || $_POST['master_key'] != hash('sha512', drupal_get_private_key())
        ) {
    httprl_fast403();
  }

  // Exit if the temp_key does not match a lock that has been taken.
  // Wait up to 5 seconds for the lock to propagate out.
  $tries = 0;
  while (lock_may_be_available($_POST['temp_key'])) {
    $tries++;
    if ($tries > 5) {
      httprl_fast403();
    }
    sleep(1);
  }

  // If request was a non blocking one, cut the connection right here.
  if (empty($_POST['mode'])) {
    httprl_background_processing('httprl async function callback running', FALSE);
  }

  // Release the lock.
  if (variable_get('lock_inc', './includes/lock.inc') !== './includes/lock.inc') {
    lock_release($_POST['temp_key']);
  }
  else {
    httprl_lock_release($_POST['temp_key']);
  }

  // Unpack arguments.
  $args = unserialize($_POST['args']);

  // Caputure anything printed out.
  ob_start();
  // Run function.
  $return = call_user_func_array($_POST['function'], $args);
  $printed = ob_get_contents();
  ob_end_clean();

  // Return data to caller.
  if (!empty($_POST['mode'])) {
    header('Content-Type: application/x-www-form-urlencoded');
    $data = array('return' => $return, 'args' => $args, 'printed' => $printed);
    echo http_build_query(array(0 => serialize($data)), '', '&');
  }

  // Exit Script.
  if (defined('VERSION') && substr(VERSION, 0, 1) >= 7) {
    drupal_exit();
  }
  else {
    exit;
  }
}

